import { List } from 'ts-toolbelt';
export declare type FunctionWithParams<P extends any[] = any[]> = (...args: P) => any;
export interface IFunctionWithMetadata<TParams extends any[]> extends FunctionWithParams<TParams> {
    __functionParamMetadata: MapMeta<TParams>;
}
export declare type ConstructorOptionalParams<T extends new (...args: any[]) => any> = new (...args: List.Optional<ConstructorParameters<T>>) => InstanceType<T>;
export declare type FunctionOptionalParams<T extends FunctionWithParams> = (...args: List.Optional<Parameters<T>>) => ReturnType<T>;
/**
 * A function to provide constructor parameter values
 * If a parameter was passed to the wrapped constructor it is passed as the first argument
 * Reflect Metadata (usually the type of the parameter) is passed as the second argument
 */
export declare type ParameterProvider = (passedParameter: any, reflectData: any, index: number) => any;
export declare type PrimitiveStringMeta<T> = T extends string ? typeof String : PrimitiveNumberMeta<T>;
export declare type PrimitiveNumberMeta<T> = T extends number ? typeof Number : PrimitiveBooleanMeta<T>;
export declare type PrimitiveBooleanMeta<T> = T extends boolean ? typeof Boolean : never;
export declare type Meta<T> = T extends string | number | boolean ? PrimitiveStringMeta<T> : new (...args: any[]) => T;
export declare type MapMeta<T extends any[]> = {
    [P in keyof T]?: Meta<T[P]>;
};
